@@ -1,6 +1,7 @@
 use crate::{
     DebugError, DebugInfo, GimliReader, ObjectRef, Variable, VariableCache, VariableLocation,
     VariableName, VariableNodeType, VariableType, VariableValue,
+    function_die::Die,
     language::{
         ProgrammingLanguage,
         value::{Value, format_float},
@@ -217,6 +218,33 @@ impl ProgrammingLanguage for Rust {
         }
     }
 
+    fn format_function_name(
+        &self,
+        function_name: &str,
+        function_die: &crate::function_die::FunctionDie<'_>,
+        debug_info: &super::DebugInfo,
+    ) -> String {
+        let parent = function_die.parent_offset();
+        if let Some(parent_offset) = parent
+            && let Ok(die) = function_die.unit_info.unit.entry(parent_offset)
+            && is_datatype(&die)
+            && let Ok(Some(typename)) = function_die.unit_info.extract_type_name(debug_info, &die)
+        {
+            // TODO: apply better heuristics to clean up the final function name
+            if let Some((_, type_generic)) = typename.split_once('<')
+                && let Some((function_without_generic, function_generic)) =
+                    function_name.split_once('<')
+                && type_generic == function_generic
+            {
+                format!("{typename}::{function_without_generic}")
+            } else {
+                format!("{typename}::{function_name}")
+            }
+        } else {
+            function_name.to_string()
+        }
+    }
+
     fn auto_resolve_children(&self, name: &str) -> bool {
         name.starts_with("&str")
             || name.starts_with("&[")
@@ -246,3 +274,7 @@ impl ProgrammingLanguage for Rust {
         Ok(())
     }
 }
+
+fn is_datatype(entry: &Die) -> bool {
+    [gimli::DW_TAG_structure_type, gimli::DW_TAG_enumeration_type].contains(&entry.tag())
+}
