@@ -23,11 +23,11 @@ impl TcpPublisher {
             }
         }
 
-        if let Some(socket) = self.socket.as_mut() {
-            if socket.write_all(bytes).is_err() {
-                // Discard socket on error. Try reconnect next time.
-                self.socket = None;
-            }
+        if let Some(socket) = self.socket.as_mut()
+            && socket.write_all(bytes).is_err()
+        {
+            // Discard socket on error. Try reconnect next time.
+            self.socket = None;
         }
     }
 }
