@@ -1,4 +1,4 @@
-use std::fmt::Display;
+use std::{fmt::Display, num::ParseIntError};
 
 use anyhow::Result;
 use jep106::JEP106Code;
@@ -36,15 +36,8 @@ pub struct Cmd {
     target_sel: Option<u32>,
 }
 
-// Clippy doesn't like `from_str_radix` with radix 10, but I prefer the symmetry`
-// with the hex case.
-#[expect(clippy::from_str_radix_10)]
-fn parse_hex(src: &str) -> Result<u32, std::num::ParseIntError> {
-    if src.starts_with("0x") {
-        u32::from_str_radix(src.trim_start_matches("0x"), 16)
-    } else {
-        u32::from_str_radix(src, 10)
-    }
+fn parse_hex(src: &str) -> Result<u32, ParseIntError> {
+    parse_int::parse(src)
 }
 
 impl Cmd {
