@@ -35,11 +35,11 @@ use crate::{
             CreateTempFileEndpoint, EraseEndpoint, FlashEndpoint, ListChipFamiliesEndpoint,
             ListProbesEndpoint, ListTestsEndpoint, LoadChipFamilyEndpoint, MonitorEndpoint,
             ProgressEventTopic, ReadMemory8Endpoint, ReadMemory16Endpoint, ReadMemory32Endpoint,
-            ReadMemory64Endpoint, ResetCoreEndpoint, ResumeAllCoresEndpoint, RpcResult,
-            RunTestEndpoint, SelectProbeEndpoint, TakeStackTraceEndpoint, TargetInfoDataTopic,
-            TargetInfoEndpoint, TempFileDataEndpoint, TokioSpawner, VerifyEndpoint,
-            WriteMemory8Endpoint, WriteMemory16Endpoint, WriteMemory32Endpoint,
-            WriteMemory64Endpoint,
+            ReadMemory64Endpoint, ResetCoreAndHaltEndpoint, ResetCoreEndpoint,
+            ResumeAllCoresEndpoint, RpcResult, RunTestEndpoint, SelectProbeEndpoint,
+            TakeStackTraceEndpoint, TargetInfoDataTopic, TargetInfoEndpoint, TempFileDataEndpoint,
+            TokioSpawner, VerifyEndpoint, WriteMemory8Endpoint, WriteMemory16Endpoint,
+            WriteMemory32Endpoint, WriteMemory64Endpoint,
             chip::{ChipData, ChipFamily, ChipInfoRequest, LoadChipFamilyRequest},
             file::{AppendFileRequest, TempFile},
             flash::{
@@ -53,7 +53,7 @@ use crate::{
                 AttachRequest, AttachResult, DebugProbeEntry, DebugProbeSelector,
                 ListProbesRequest, SelectProbeRequest, SelectProbeResult,
             },
-            reset::ResetCoreRequest,
+            reset::{ResetCoreAndHaltRequest, ResetCoreRequest},
             resume::ResumeAllCoresRequest,
             rtt_client::{CreateRttClientRequest, RttClientData, ScanRegion},
             stack_trace::{StackTraces, TakeStackTraceRequest},
@@ -85,9 +85,10 @@ pub async fn connect(host: &str, token: Option<String>) -> anyhow::Result<RpcCli
 
     let uri = Uri::from_str(&format!("{host}/worker")).context("Failed to parse server URI")?;
 
-    let is_localhost = uri
-        .host()
-        .is_some_and(|h| ["localhost", "127.0.0.1", "::1"].contains(&h));
+    // We could check the host address for localhost and then set the `is_localhost` option, but
+    // there are setups where the user uses port forwarding and the file actually needs to be
+    // uploaded for correct behavior. Therefore, this check is not performed.
+
     let req = ClientRequestBuilder::new(uri).with_header(
         "User-Agent",
         format!("probe-rs-tools {}", env!("PROBE_RS_LONG_VERSION")),
@@ -132,7 +133,7 @@ pub async fn connect(host: &str, token: Option<String>) -> anyhow::Result<RpcCli
         .await
         .map_err(|err| anyhow::anyhow!("Failed to send challenge response: {err:?}"))?;
 
-    let mut client = RpcClient::new_from_wire(
+    Ok(RpcClient::new_from_wire(
         tx,
         WebsocketRx::new(rx.map(|message| {
             message.map(|message| match message {
@@ -140,10 +141,7 @@ pub async fn connect(host: &str, token: Option<String>) -> anyhow::Result<RpcCli
                 _ => Bytes::new(),
             })
         })),
-    );
-    client.is_localhost = is_localhost;
-
-    Ok(client)
+    ))
 }
 
 #[cfg(feature = "remote")]
@@ -251,6 +249,10 @@ impl RpcClient {
         }
     }
 
+    pub fn is_local_session(&self) -> bool {
+        self.is_localhost
+    }
+
     pub fn new_local_from_wire(
         tx: impl PostcardSender + Send + Sync + 'static,
         rx: impl PostcardReceiver + Send + 'static,
@@ -444,6 +446,8 @@ impl SessionInterface {
         &self,
         mut path: PathBuf,
         mut format: FormatOptions,
+        image_target: Option<String>,
+        read_flasher_rtt: bool,
     ) -> anyhow::Result<BuildResult> {
         path = self.client.upload_file(&path).await?;
 
@@ -470,6 +474,8 @@ impl SessionInterface {
                 sessid: self.sessid,
                 path: path.display().to_string(),
                 format,
+                image_target,
+                read_flasher_rtt,
             })
             .await
     }
@@ -497,6 +503,7 @@ impl SessionInterface {
     pub async fn erase(
         &self,
         command: EraseCommand,
+        read_flasher_rtt: bool,
         on_msg: impl AsyncFnMut(ProgressEvent),
     ) -> anyhow::Result<()> {
         self.client
@@ -504,6 +511,7 @@ impl SessionInterface {
                 &EraseRequest {
                     sessid: self.sessid,
                     command,
+                    read_flasher_rtt,
                 },
                 on_msg,
             )
@@ -582,13 +590,18 @@ impl SessionInterface {
             .await
     }
 
-    pub async fn stack_trace(&self, path: PathBuf) -> anyhow::Result<StackTraces> {
+    pub async fn stack_trace(
+        &self,
+        path: PathBuf,
+        stack_frame_limit: u32,
+    ) -> anyhow::Result<StackTraces> {
         let path = self.client.upload_file(&path).await?;
 
         self.client
             .send_resp::<TakeStackTraceEndpoint, _>(&TakeStackTraceRequest {
                 sessid: self.sessid,
                 path: path.display().to_string(),
+                stack_frame_limit,
             })
             .await
     }
@@ -708,6 +721,16 @@ impl CoreInterface {
             })
             .await
     }
+
+    pub async fn reset_and_halt(&self, timeout: Duration) -> anyhow::Result<()> {
+        self.client
+            .send_resp::<ResetCoreAndHaltEndpoint, _>(&ResetCoreAndHaltRequest {
+                sessid: self.sessid,
+                core: self.core,
+                timeout,
+            })
+            .await
+    }
 }
 
 #[derive(Debug)]
