@@ -134,7 +134,7 @@ fn list_tests_impl(
     request: ListTestsRequest,
     sender: MonitorSender,
 ) -> anyhow::Result<Tests> {
-    let mut session = ctx.session_blocking(request.sessid);
+    let shared_session = ctx.shared_session(request.sessid);
     let mut list_handler = ListEventHandler::new(request.semihosting_options, |event| {
         sender.send_semihosting_event(event).unwrap()
     });
@@ -150,15 +150,14 @@ fn list_tests_impl(
         cancellation_token: ctx.cancellation_token(),
     };
 
-    request.boot_info.prepare(&mut session, run_loop.core_id)?;
-
-    let mut core = session.core(0)?;
-    if let Some(rtt_client) = rtt_client.as_mut() {
-        rtt_client.clear_control_block(&mut core)?;
+    {
+        let mut session = shared_session.session_blocking();
+        request.boot_info.prepare(&mut session, run_loop.core_id)?;
     }
 
     let poller = rtt_client.as_deref_mut().map(|client| RttPoller {
         rtt_client: client,
+        clear_control_block: true,
         sender: |message| {
             sender
                 .send_rtt_event(message)
@@ -167,7 +166,7 @@ fn list_tests_impl(
     });
 
     match run_loop.run_until(
-        &mut core,
+        &shared_session,
         true,
         true,
         poller,
@@ -226,18 +225,18 @@ fn run_test_impl(
     let timeout = request.test.timeout.map(|t| Duration::from_secs(t as u64));
     let timeout = timeout.unwrap_or(Duration::from_secs(60));
 
-    let mut session = ctx.session_blocking(request.sessid);
+    let shared_session = ctx.shared_session(request.sessid);
 
     let mut rtt_client = request
         .rtt_client
         .map(|rtt_client| ctx.object_mut_blocking(rtt_client));
 
     let core_id = rtt_client.as_ref().map(|rtt| rtt.core_id()).unwrap_or(0);
-    let mut core = session.core(core_id)?;
-    core.reset_and_halt(Duration::from_millis(100))?;
 
-    if let Some(rtt_client) = rtt_client.as_mut() {
-        rtt_client.clear_control_block(&mut core)?;
+    {
+        let mut session = shared_session.session_blocking();
+        let mut core = session.core(core_id)?;
+        core.reset_and_halt(Duration::from_millis(100))?;
     }
 
     let expected_outcome = request.test.expected_outcome;
@@ -253,6 +252,7 @@ fn run_test_impl(
 
     let poller = rtt_client.as_deref_mut().map(|client| RttPoller {
         rtt_client: client,
+        clear_control_block: true,
         sender: |message| {
             sender
                 .send_rtt_event(message)
@@ -261,7 +261,7 @@ fn run_test_impl(
     });
 
     match run_loop.run_until(
-        &mut core,
+        &shared_session,
         true,
         true,
         poller,
