@@ -42,6 +42,21 @@ pub struct BinaryDownloadOptions {
     /// Whether to erase the entire chip before downloading
     #[arg(long, help_heading = "DOWNLOAD CONFIGURATION")]
     pub chip_erase: bool,
+
+    /// Whether to read the RTT output from the flash loader, if available.
+    #[arg(long, help_heading = "DOWNLOAD CONFIGURATION")]
+    pub read_flasher_rtt: bool,
+    /// The preferred flash algorithms for specific memory regions can be overriden.
+    ///
+    /// Multiple algorithms can be specified as a comma-separated list, e.g. --prefer-flash-algorithm=algo1,algo2
+    #[arg(
+        long,
+        env = "PROBE_RS_PREFER_FLASH_ALGO",
+        value_delimiter = ',',
+        num_args = 1..,
+        help_heading = "PROBE CONFIGURATION"
+    )]
+    pub prefer_flash_algorithm: Vec<String>,
 }
 
 /// Supported bit-widths for read/write commands (not every device may support each width).
@@ -111,6 +126,7 @@ pub struct ProbeOptions {
         help_heading = "PROBE CONFIGURATION"
     )]
     pub connect_under_reset: bool,
+
     #[arg(long, env = "PROBE_RS_DRY_RUN", help_heading = "PROBE CONFIGURATION")]
     pub dry_run: bool,
     /// Use this flag to allow all memory, including security keys and 3rd party
@@ -527,12 +543,6 @@ pub enum OperationError {
     #[error("Failed to get a handle to the first core.")]
     AttachingToCoreFailed(#[source] probe_rs::Error),
 
-    #[error("The reset of the target failed.")]
-    TargetResetFailed(#[source] probe_rs::Error),
-
-    #[error("The target could not be reset and halted.")]
-    TargetResetHaltFailed(#[source] probe_rs::Error),
-
     #[error("Failed to write to file")]
     IOError(#[source] std::io::Error),
 
@@ -540,6 +550,9 @@ pub enum OperationError {
     CliArgument(#[from] clap::Error),
     #[error("Failed to parse interactive probe index selection")]
     ParseProbeIndex(#[source] std::num::ParseIntError),
+
+    #[error(transparent)]
+    Anyhow(#[from] anyhow::Error),
 }
 
 /// Used in errors it can print a list of items.
