@@ -74,17 +74,16 @@ fn armv7a_reset_catch_set(
     core: &mut dyn ArmMemoryInterface,
     debug_base: Option<u64>,
 ) -> Result<(), ArmError> {
-    use crate::architecture::arm::core::armv7a_debug_regs::Dbgprcr;
+    use crate::architecture::arm::core::armv7a_debug_regs::Dbgvcr;
 
     let debug_base =
         debug_base.ok_or_else(|| ArmError::from(ArmDebugSequenceError::DebugBaseNotSpecified))?;
 
-    let address = Dbgprcr::get_mmio_address_from_base(debug_base)?;
-    let mut dbgprcr = Dbgprcr(core.read_word_32(address)?);
-
-    dbgprcr.set_hcwr(true);
-
-    core.write_word_32(address, dbgprcr.into())?;
+    tracing::debug!("Enabling reset vector catch");
+    let address = Dbgvcr::get_mmio_address_from_base(debug_base)?;
+    let mut dbgvcr = Dbgvcr(core.read_word_32(address)?);
+    dbgvcr.set_r(true);
+    core.write_word_32(address, dbgvcr.into())?;
 
     Ok(())
 }
@@ -94,17 +93,16 @@ fn armv7a_reset_catch_clear(
     core: &mut dyn ArmMemoryInterface,
     debug_base: Option<u64>,
 ) -> Result<(), ArmError> {
-    use crate::architecture::arm::core::armv7a_debug_regs::Dbgprcr;
+    use crate::architecture::arm::core::armv7a_debug_regs::Dbgvcr;
 
     let debug_base =
         debug_base.ok_or_else(|| ArmError::from(ArmDebugSequenceError::DebugBaseNotSpecified))?;
 
-    let address = Dbgprcr::get_mmio_address_from_base(debug_base)?;
-    let mut dbgprcr = Dbgprcr(core.read_word_32(address)?);
-
-    dbgprcr.set_hcwr(false);
-
-    core.write_word_32(address, dbgprcr.into())?;
+    tracing::debug!("Disabling reset vector catch");
+    let address = Dbgvcr::get_mmio_address_from_base(debug_base)?;
+    let mut dbgvcr = Dbgvcr(core.read_word_32(address)?);
+    dbgvcr.set_r(false);
+    core.write_word_32(address, dbgvcr.into())?;
 
     Ok(())
 }
@@ -113,6 +111,11 @@ fn armv7a_reset_system(
     interface: &mut dyn ArmMemoryInterface,
     debug_base: Option<u64>,
 ) -> Result<(), ArmError> {
+    // Note that this is a best-effort reset that might not do anything;
+    // per Arm docs for EDPRCR, the request is implementation-defined and
+    // Arm deprecate use of this bit. You may need to implement a vendor-specific
+    // reset sequence instead.
+    tracing::debug!("Running default ARMv7A system reset via DBGPRCR.CWRR");
     use crate::architecture::arm::core::armv7a_debug_regs::{Dbgprcr, Dbgprsr};
 
     let debug_base =
@@ -144,7 +147,9 @@ fn armv7a_core_start(
     core: &mut dyn ArmMemoryInterface,
     debug_base: Option<u64>,
 ) -> Result<(), ArmError> {
-    use crate::architecture::arm::core::armv7a_debug_regs::{Dbgdsccr, Dbgdscr, Dbgdsmcr, Dbglar};
+    use crate::architecture::arm::core::armv7a_debug_regs::{
+        Dbgdsccr, Dbgdscr, Dbgdsmcr, Dbglar, Dbgvcr,
+    };
 
     let debug_base =
         debug_base.ok_or_else(|| ArmError::from(ArmDebugSequenceError::DebugBaseNotSpecified))?;
@@ -165,6 +170,10 @@ fn armv7a_core_start(
     let address = Dbgdsmcr::get_mmio_address_from_base(debug_base)?;
     core.write_word_32(address, Dbgdsmcr(0).into())?;
 
+    // Clear all vector catch bits to ensure defined startup value
+    let address = Dbgvcr::get_mmio_address_from_base(debug_base)?;
+    core.write_word_32(address, Dbgvcr(0).into())?;
+
     // Enable halting
     let address = Dbgdscr::get_mmio_address_from_base(debug_base)?;
     let mut dbgdscr = Dbgdscr(core.read_word_32(address)?);
@@ -225,6 +234,10 @@ fn armv8a_reset_system(
     interface: &mut dyn ArmMemoryInterface,
     debug_base: Option<u64>,
 ) -> Result<(), ArmError> {
+    // Note that this is a best-effort reset that might not do anything;
+    // per Arm docs for EDPRCR, the request is implementation-defined and
+    // Arm deprecate use of this bit. You may need to implement a vendor-specific
+    // reset sequence instead.
     use crate::architecture::arm::core::armv8a_debug_regs::{Edprcr, Edprsr};
 
     let debug_base =
@@ -372,7 +385,9 @@ fn cortex_m_reset_catch_set(core: &mut dyn ArmMemoryInterface) -> Result<(), Arm
 }
 
 /// ResetSystem for Cortex-M devices
-fn cortex_m_reset_system(interface: &mut dyn ArmMemoryInterface) -> Result<(), ArmError> {
+pub(crate) fn cortex_m_reset_system(
+    interface: &mut dyn ArmMemoryInterface,
+) -> Result<(), ArmError> {
     use crate::architecture::arm::core::armv7m::Aircr;
 
     let mut aircr = Aircr(0);
