@@ -3,7 +3,6 @@
 use std::{sync::Arc, time::Duration};
 
 use probe_rs_target::{Architecture, CoreType, InstructionSet};
-use zerocopy::IntoBytes;
 
 use crate::{
     CoreInformation, CoreInterface, CoreRegister, CoreStatus, Error, HaltReason, MemoryInterface,
@@ -23,6 +22,7 @@ use crate::{
         BreakpointCause,
         registers::{CoreRegisters, RegisterId, RegisterValue},
     },
+    memory::CoreMemoryInterface,
     semihosting::{SemihostingCommand, decode_semihosting_syscall},
 };
 
@@ -54,9 +54,6 @@ pub struct XtensaCoreState {
 
     /// The semihosting command that was decoded at the current program counter
     semihosting_command: Option<SemihostingCommand>,
-
-    /// Whether the registers have been spilled to the stack.
-    spilled: bool,
 }
 
 impl XtensaCoreState {
@@ -68,7 +65,6 @@ impl XtensaCoreState {
             breakpoint_set: [false; 2],
             pc_written: false,
             semihosting_command: None,
-            spilled: false,
         }
     }
 
@@ -110,7 +106,6 @@ impl<'probe> Xtensa<'probe> {
     }
 
     fn clear_cache(&mut self) {
-        self.state.spilled = false;
         self.interface.clear_register_cache();
     }
 
@@ -300,11 +295,22 @@ impl<'probe> Xtensa<'probe> {
     }
 
     fn spill_registers(&mut self) -> Result<(), Error> {
-        if self.state.spilled {
+        if self.current_ps()?.excm() {
+            // We are in an exception, possibly WindowOverflowN or WindowUnderflowN.
+            // We can't spill registers in this state.
             return Ok(());
         }
-        self.state.spilled = true;
+        if self.current_ps()?.woe() {
+            let register_file = self.read_window_registers()?;
+            // We should only spill registers if PS.WOE is set. According to the debug guide, we
+            // also should not spill if INTLEVEL != 0 but I don't see why.
+            register_file.spill(&mut self.interface)?;
+        }
 
+        Ok(())
+    }
+
+    fn read_window_registers(&mut self) -> Result<RegisterFile, Error> {
         let register_file = RegisterFile::read(
             self.interface.core_properties().window_option_properties,
             &mut self.interface,
@@ -320,123 +326,18 @@ impl<'probe> Xtensa<'probe> {
                 .register_cache
                 .store(Register::Cpu(reg), value);
         }
-
-        if self.current_ps()?.excm() {
-            // We are in an exception, possibly WindowOverflowN or WindowUnderflowN.
-            // We can't spill registers in this state.
-            return Ok(());
-        }
-        if self.current_ps()?.woe() {
-            // We should only spill registers if PS.WOE is set. According to the debug guide, we
-            // also should not spill if INTLEVEL != 0 but I don't see why.
-            register_file.spill(&mut self.interface)?;
-        }
-
-        Ok(())
+        Ok(register_file)
     }
 }
 
-// We can't use CoreMemoryInterface here, because we need to spill registers before reading.
-// This needs to be considerably cleaned up.
-impl MemoryInterface for Xtensa<'_> {
-    fn supports_native_64bit_access(&mut self) -> bool {
-        self.interface.supports_native_64bit_access()
+impl CoreMemoryInterface for Xtensa<'_> {
+    type ErrorType = Error;
+
+    fn memory(&self) -> &dyn MemoryInterface<Self::ErrorType> {
+        &self.interface
     }
-
-    fn read_word_64(&mut self, address: u64) -> Result<u64, Error> {
-        self.halted_access(|this| {
-            this.spill_registers()?;
-
-            this.interface.read_word_64(address)
-        })
-    }
-
-    fn read_word_32(&mut self, address: u64) -> Result<u32, Error> {
-        self.halted_access(|this| {
-            this.spill_registers()?;
-
-            this.interface.read_word_32(address)
-        })
-    }
-
-    fn read_word_16(&mut self, address: u64) -> Result<u16, Error> {
-        self.halted_access(|this| {
-            this.spill_registers()?;
-
-            this.interface.read_word_16(address)
-        })
-    }
-
-    fn read_word_8(&mut self, address: u64) -> Result<u8, Error> {
-        self.halted_access(|this| {
-            this.spill_registers()?;
-
-            this.interface.read_word_8(address)
-        })
-    }
-
-    fn read_64(&mut self, address: u64, data: &mut [u64]) -> Result<(), Error> {
-        self.read_8(address, data.as_mut_bytes())
-    }
-
-    fn read_32(&mut self, address: u64, data: &mut [u32]) -> Result<(), Error> {
-        self.read_8(address, data.as_mut_bytes())
-    }
-
-    fn read_16(&mut self, address: u64, data: &mut [u16]) -> Result<(), Error> {
-        self.read_8(address, data.as_mut_bytes())
-    }
-
-    fn read_8(&mut self, address: u64, data: &mut [u8]) -> Result<(), Error> {
-        self.halted_access(|this| {
-            this.spill_registers()?;
-
-            this.interface.read_8(address, data)
-        })
-    }
-
-    fn write_word_64(&mut self, address: u64, data: u64) -> Result<(), Error> {
-        self.interface.write_word_64(address, data)
-    }
-
-    fn write_word_32(&mut self, address: u64, data: u32) -> Result<(), Error> {
-        self.interface.write_word_32(address, data)
-    }
-
-    fn write_word_16(&mut self, address: u64, data: u16) -> Result<(), Error> {
-        self.interface.write_word_16(address, data)
-    }
-
-    fn write_word_8(&mut self, address: u64, data: u8) -> Result<(), Error> {
-        self.interface.write_word_8(address, data)
-    }
-
-    fn write_64(&mut self, address: u64, data: &[u64]) -> Result<(), Error> {
-        self.interface.write_64(address, data)
-    }
-
-    fn write_32(&mut self, address: u64, data: &[u32]) -> Result<(), Error> {
-        self.interface.write_32(address, data)
-    }
-
-    fn write_16(&mut self, address: u64, data: &[u16]) -> Result<(), Error> {
-        self.interface.write_16(address, data)
-    }
-
-    fn write_8(&mut self, address: u64, data: &[u8]) -> Result<(), Error> {
-        self.interface.write_8(address, data)
-    }
-
-    fn write(&mut self, address: u64, data: &[u8]) -> Result<(), Error> {
-        self.interface.write(address, data)
-    }
-
-    fn supports_8bit_transfers(&self) -> Result<bool, Error> {
-        self.interface.supports_8bit_transfers()
-    }
-
-    fn flush(&mut self) -> Result<(), Error> {
-        self.interface.flush()
+    fn memory_mut(&mut self) -> &mut dyn MemoryInterface<Self::ErrorType> {
+        &mut self.interface
     }
 }
 
@@ -677,6 +578,10 @@ impl CoreInterface for Xtensa<'_> {
         self.interface.leave_debug_mode()?;
         Ok(())
     }
+
+    fn spill_registers(&mut self) -> Result<(), Error> {
+        self.spill_registers()
+    }
 }
 
 struct RegisterFile {
@@ -694,13 +599,14 @@ impl RegisterFile {
         let window_base_result = interface.schedule_read_register(SpecialRegister::Windowbase)?;
         let window_start_result = interface.schedule_read_register(SpecialRegister::Windowstart)?;
 
-        let mut register_results = Vec::with_capacity(xtensa.num_aregs as usize);
+        let mut register_values = Vec::with_capacity(xtensa.num_aregs as usize);
 
         let ar0 = arch::CpuRegister::A0 as u8;
 
         // Restore registers before reading them, as reading a special
         // register overwrote scratch registers.
         interface.restore_registers()?;
+
         // The window registers alias each other, so we need to make sure we don't read
         // the cached values. We'll then use the registers we read here to prime the cache.
         for ar in ar0..ar0 + xtensa.window_regs {
@@ -708,13 +614,13 @@ impl RegisterFile {
             interface.state.register_cache.remove(reg.into());
         }
 
+        let mut regs = Vec::with_capacity(xtensa.window_regs as usize);
         for _ in 0..xtensa.num_aregs / xtensa.window_regs {
             // Read registers visible in the current window
             for ar in ar0..ar0 + xtensa.window_regs {
                 let reg = CpuRegister::try_from(ar)?;
-                let result = interface.schedule_read_register(reg)?;
-
-                register_results.push(result);
+                let deferred_result = interface.schedule_read_register(reg)?;
+                regs.push(deferred_result);
             }
 
             // Rotate window to see the next `window_regs` registers
@@ -722,6 +628,19 @@ impl RegisterFile {
             interface
                 .xdm
                 .schedule_execute_instruction(Instruction::Rotw(rotw_arg));
+
+            // Pull out values from cache before we clear the cache
+            for deferred in regs.drain(..) {
+                let result = interface.read_deferred_result(deferred)?;
+                register_values.push(result);
+            }
+
+            // The window registers alias each other, so we need to make sure we don't read
+            // the cached values. We'll then use the registers we read here to prime the cache.
+            for ar in ar0..ar0 + xtensa.window_regs {
+                let reg = CpuRegister::try_from(ar)?;
+                interface.state.register_cache.remove(reg.into());
+            }
         }
 
         // Now do the actual read.
@@ -730,11 +649,6 @@ impl RegisterFile {
             .execute()
             .expect("Failed to execute read. This shouldn't happen.");
 
-        let mut register_values = register_results
-            .into_iter()
-            .map(|result| interface.read_deferred_result(result))
-            .collect::<Result<Vec<_>, _>>()?;
-
         // WindowBase points to the first register of the current window in the register file.
         // In essence, it selects which 16 registers are visible out of the 64 physical registers.
         let window_base = interface.read_deferred_result(window_base_result)?;
