@@ -90,7 +90,7 @@ pub enum DebugRegisterError {
 
 #[derive(thiserror::Error, Debug, Clone, Copy, docsplay::Display)]
 pub enum Error {
-    /// Error {access} register {narsel:#04X}
+    /// Error {access} {print_narsel(narsel)} register
     Xdm {
         /// Nexus Address Register selector. Contains the ID of the register being accessed.
         narsel: u8,
@@ -103,7 +103,7 @@ pub enum Error {
     },
 
     /// The instruction execution has encountered an exception.
-    ExecExeception,
+    ExecException,
 
     /// The core is still executing a previous instruction.
     ExecBusy,
@@ -119,6 +119,21 @@ pub enum Error {
     XdmPoweredOff,
 }
 
+fn print_narsel(narsel: &u8) -> String {
+    let name = match *narsel {
+        0x40 => "the OCDID",
+        0x42 => "the DCRCLR",
+        0x43 => "the DCRSET",
+        0x44 => "the DSR",
+        0x45 => "the DDR",
+        0x46 => "the DDREXEC",
+        0x47 => "the DIR0EXEC",
+        0x48 => "the DIR0",
+        _ => "an unknown",
+    };
+    format!("{name} ({narsel:#04X})")
+}
+
 #[derive(Debug, Default)]
 pub struct XdmState {
     /// The last instruction to be executed.
@@ -309,7 +324,9 @@ impl<'probe> Xdm<'probe> {
                     return Ok(());
                 }
                 Err(e) => {
-                    let mut to_consume = e.results.len();
+                    // Consume the successful commands.
+                    queue.consume(e.results.len());
+
                     match e.error {
                         ProbeRsError::Xtensa(XtensaError::XdmError(Error::Xdm {
                             source: DebugRegisterError::Busy,
@@ -320,15 +337,28 @@ impl<'probe> Xdm<'probe> {
                         }
                         ProbeRsError::Xtensa(XtensaError::XdmError(Error::ExecBusy)) => {
                             // The instruction is still executing. Retry the Debug Status read.
-                            to_consume -= 1;
+                            // Do not rewind the queue, it will not include the failed command.
+                            // Retrying the failed command will just shift the DSR out of the
+                            // JTAG DR again, re-attempting the read.
                         }
-                        ProbeRsError::Xtensa(XtensaError::XdmError(Error::ExecExeception)) => {
+                        ProbeRsError::Xtensa(XtensaError::XdmError(Error::ExecOverrun)) => {
+                            // Clear exception/overrun to allow executing further instructions.
+                            self.clear_exception_state()?;
+                            // This state is reported by a DebugStatus read. We need to rewind this read and
+                            // the previous commands that attempted to execute an instruction. Commands come in NAR-NDR
+                            // pairs, and we don't have to rewind the last NDR, because the failed instruction is not
+                            // part of the returned results, hence it is not consumed.
+                            if !queue.rewind(3) {
+                                return Err(XtensaError::XdmError(Error::ExecOverrun));
+                            }
+                        }
+                        ProbeRsError::Xtensa(XtensaError::XdmError(Error::ExecException)) => {
                             // Clear exception to allow executing further instructions.
                             self.clear_exception_state()?;
                             // TODO: in the future, we might want to bubble up the exception cause.
                             // We might also want to store this error for each result that has not
                             // yet been read.
-                            return Err(XtensaError::XdmError(Error::ExecExeception));
+                            return Err(XtensaError::XdmError(Error::ExecException));
                         }
 
                         ProbeRsError::Probe(error) => return Err(error.into()),
@@ -336,8 +366,6 @@ impl<'probe> Xdm<'probe> {
                         other => panic!("Unexpected error: {other}"),
                     }
 
-                    // queue up the remaining commands when we retry
-                    queue.consume(to_consume);
                     self.state.jtag_results.merge_from(e.results);
                 }
             }
@@ -672,7 +700,6 @@ impl<'probe> Xdm<'probe> {
             let mut status = DebugStatus(0);
 
             status.set_exec_exception(true);
-            status.set_exec_done(true);
             status.set_exec_overrun(true);
 
             status
@@ -709,7 +736,7 @@ fn transform_instruction_status(
     }
     if status.exec_exception() {
         return Err(ProbeRsError::Xtensa(XtensaError::XdmError(
-            Error::ExecExeception,
+            Error::ExecException,
         )));
     }
     if status.exec_busy() {
