@@ -110,7 +110,7 @@ where
         // to bytes. Assumes target is little endian. May be overridden to
         // provide an implementation that avoids heap allocation and endian
         // conversions. Must be overridden for big endian targets.
-        if data.len() % 8 != 0 {
+        if !data.len().is_multiple_of(8) {
             return Err(InvalidDataLengthError::new("read_mem_64bit", 8).into());
         }
         let mut buffer = vec![0u64; data.len() / 8];
@@ -130,7 +130,7 @@ where
         // to bytes. Assumes target is little endian. May be overridden to
         // provide an implementation that avoids heap allocation and endian
         // conversions. Must be overridden for big endian targets.
-        if data.len() % 4 != 0 {
+        if !data.len().is_multiple_of(4) {
             return Err(InvalidDataLengthError::new("read_mem_32bit", 4).into());
         }
         let mut buffer = vec![0u32; data.len() / 4];
@@ -155,7 +155,7 @@ where
         if self.supports_native_64bit_access() {
             // Avoid heap allocation and copy if we don't need it.
             self.read_8(address, data)?;
-        } else if address % 4 == 0 && data.len() % 4 == 0 {
+        } else if address.is_multiple_of(4) && data.len().is_multiple_of(4) {
             // Avoid heap allocation and copy if we don't need it.
             self.read_mem_32bit(address, data)?;
         } else {
@@ -227,7 +227,7 @@ where
         // to bytes. Assumes target is little endian. May be overridden to
         // provide an implementation that avoids heap allocation and endian
         // conversions. Must be overridden for big endian targets.
-        if data.len() % 8 != 0 {
+        if !data.len().is_multiple_of(8) {
             return Err(InvalidDataLengthError::new("write_mem_64bit", 8).into());
         }
         let mut buffer = vec![0u64; data.len() / 8];
@@ -248,7 +248,7 @@ where
         // to bytes. Assumes target is little endian. May be overridden to
         // provide an implementation that avoids heap allocation and endian
         // conversions. Must be overridden for big endian targets.
-        if data.len() % 4 != 0 {
+        if !data.len().is_multiple_of(4) {
             return Err(InvalidDataLengthError::new("write_mem_32bit", 4).into());
         }
         let mut buffer = vec![0u32; data.len() / 4];
@@ -275,7 +275,7 @@ where
         let inbetween_count = len - start_extra_count - end_extra_count;
         assert!(start_extra_count < 4);
         assert!(end_extra_count < 4);
-        assert!(inbetween_count % 4 == 0);
+        assert!(inbetween_count.is_multiple_of(4));
 
         if start_extra_count != 0 || end_extra_count != 0 {
             // If we do not support 8 bit transfers we have to bail
@@ -329,6 +329,56 @@ where
     /// can be called.  Takes no arguments, but may return failure if a batched
     /// operation fails.
     fn flush(&mut self) -> Result<(), ERR>;
+
+    /// Execute a single memory operation.
+    ///
+    /// This function is used to execute a single memory operation, such as reading or writing data.
+    fn execute_single_memory_operation(
+        &mut self,
+        mut operation: Operation<'_>,
+    ) -> Result<(), crate::Error>
+    where
+        Error: From<ERR>,
+    {
+        let result = match operation.operation {
+            OperationKind::Read(ref mut data) => self.read(operation.address, data),
+            OperationKind::Read8(ref mut data) => self.read_8(operation.address, data),
+            OperationKind::Read16(ref mut data) => self.read_16(operation.address, data),
+            OperationKind::Read32(ref mut data) => self.read_32(operation.address, data),
+            OperationKind::Read64(ref mut data) => self.read_64(operation.address, data),
+            OperationKind::Write(data) => self.write(operation.address, data),
+            OperationKind::Write8(data) => self.write_8(operation.address, data),
+            OperationKind::Write16(data) => self.write_16(operation.address, data),
+            OperationKind::Write32(data) => self.write_32(operation.address, data),
+            OperationKind::Write64(data) => self.write_64(operation.address, data),
+            OperationKind::WriteWord8(data) => self.write_word_8(operation.address, data),
+            OperationKind::WriteWord16(data) => self.write_word_16(operation.address, data),
+            OperationKind::WriteWord32(data) => self.write_word_32(operation.address, data),
+            OperationKind::WriteWord64(data) => self.write_word_64(operation.address, data),
+        };
+        result.map_err(Error::from)
+    }
+
+    /// Execute a batch of operations.
+    ///
+    /// Operations are executed in the order they are provided.
+    /// If any operation fails, the remaining operations are not executed.
+    /// The result of each operation is stored in the `result` field of the `Operation` struct.
+    ///
+    /// This method should be implemented for architectures that don't support background memory access.
+    fn execute_memory_operations(&mut self, operations: &mut [Operation<'_>])
+    where
+        Error: From<ERR>,
+    {
+        for operation in operations {
+            let result = self.execute_single_memory_operation(operation.reborrow());
+            let success = result.is_ok();
+            operation.result = Some(result);
+            if !success {
+                break;
+            }
+        }
+    }
 }
 
 // Helper functions to validate address space constraints
@@ -337,7 +387,7 @@ where
 pub(crate) fn valid_32bit_address(address: u64) -> Result<u32, Error> {
     let address: u32 = address
         .try_into()
-        .map_err(|_| Error::Other(format!("Address {:#08x} out of range", address)))?;
+        .map_err(|_| Error::Other(format!("Address {address:#08x} out of range")))?;
 
     Ok(address)
 }
@@ -463,4 +513,76 @@ where
     fn flush(&mut self) -> Result<(), Error> {
         self.memory_mut().flush().map_err(Error::from)
     }
+
+    fn execute_memory_operations(&mut self, operations: &mut [Operation<'_>]) {
+        self.memory_mut().execute_memory_operations(operations)
+    }
+}
+
+/// The kind of memory operation that can be batched.
+#[derive(Debug)]
+pub enum OperationKind<'a> {
+    Read(&'a mut [u8]),
+    Read8(&'a mut [u8]),
+    Read16(&'a mut [u16]),
+    Read32(&'a mut [u32]),
+    Read64(&'a mut [u64]),
+    Write(&'a [u8]),
+    Write8(&'a [u8]),
+    Write16(&'a [u16]),
+    Write32(&'a [u32]),
+    Write64(&'a [u64]),
+    WriteWord8(u8),
+    WriteWord16(u16),
+    WriteWord32(u32),
+    WriteWord64(u64),
+}
+
+/// A memory operation that can be batched.
+#[derive(Debug)]
+pub struct Operation<'a> {
+    /// The address of the operation.
+    pub address: u64,
+
+    /// The result of the operation.
+    ///
+    /// If the operation has not been executed yet, this will be `None`. Otherwise, it will contain the result of the operation.
+    pub result: Option<Result<(), Error>>,
+
+    /// The kind of operation.
+    pub operation: OperationKind<'a>,
+}
+
+impl<'a> Operation<'a> {
+    /// Create a new memory operation.
+    pub fn new(address: u64, operation: OperationKind<'a>) -> Self {
+        Operation {
+            address,
+            result: None,
+            operation,
+        }
+    }
+
+    pub(crate) fn reborrow(&mut self) -> Operation<'_> {
+        Operation {
+            address: self.address,
+            result: self.result.take(),
+            operation: match self.operation {
+                OperationKind::Read(ref mut data) => OperationKind::Read(data),
+                OperationKind::Read8(ref mut data) => OperationKind::Read8(data),
+                OperationKind::Read16(ref mut data) => OperationKind::Read16(data),
+                OperationKind::Read32(ref mut data) => OperationKind::Read32(data),
+                OperationKind::Read64(ref mut data) => OperationKind::Read64(data),
+                OperationKind::Write(data) => OperationKind::Write(data),
+                OperationKind::Write8(data) => OperationKind::Write8(data),
+                OperationKind::Write16(data) => OperationKind::Write16(data),
+                OperationKind::Write32(data) => OperationKind::Write32(data),
+                OperationKind::Write64(data) => OperationKind::Write64(data),
+                OperationKind::WriteWord8(data) => OperationKind::WriteWord8(data),
+                OperationKind::WriteWord16(data) => OperationKind::WriteWord16(data),
+                OperationKind::WriteWord32(data) => OperationKind::WriteWord32(data),
+                OperationKind::WriteWord64(data) => OperationKind::WriteWord64(data),
+            },
+        }
+    }
 }
