@@ -20,7 +20,7 @@ use crate::{
     },
 };
 use bitvec::prelude::*;
-use nusb::DeviceInfo;
+use nusb::{DeviceInfo, MaybeFuture};
 use std::{
     io::{Read, Write},
     sync::Arc,
@@ -98,6 +98,8 @@ impl JtagAdapter {
             (0x0403, 0x6014, "Digilent Adept USB Device") => (0x00e8, 0x60eb),
             // Digilent HS1
             (0x0403, 0x6010, "Digilent Adept USB Device") => (0x0088, 0x008b),
+            // Built-in Digilent HS1 (on-board)
+            (0x0403, 0x6010, "Digilent USB Device") => (0x0088, 0x008b),
             // Other devices:
             // TMS starts high
             // TMS, TDO and TCK are outputs
@@ -281,8 +283,11 @@ impl ProbeFactory for FtdiProbeFactory {
             ));
         };
 
-        let mut probes = nusb::list_devices()
-            .map_err(FtdiError::from)?
+        let devices = nusb::list_devices()
+            .wait()
+            .map_err(|e| DebugProbeError::from(FtdiError::Usb(e.into())))?;
+
+        let mut probes = devices
             .filter(|usb_info| selector.matches(usb_info))
             .collect::<Vec<_>>();
 
@@ -603,17 +608,21 @@ fn get_device_info(device: &DeviceInfo) -> Option<DebugProbeInfo> {
             product_id: device.product_id(),
             serial_number: device.serial_number().map(|s| s.to_string()),
             probe_factory: &FtdiProbeFactory,
-            hid_interface: None,
+            is_hid_interface: false,
+            interface: None,
         })
     })
 }
 
 #[tracing::instrument(skip_all)]
 fn list_ftdi_devices() -> Vec<DebugProbeInfo> {
-    match nusb::list_devices() {
+    match nusb::list_devices().wait() {
         Ok(devices) => devices
             .filter_map(|device| get_device_info(&device))
             .collect(),
-        Err(_) => vec![],
+        Err(e) => {
+            tracing::warn!("error listing FTDI devices: {e}");
+            vec![]
+        }
     }
 }
