@@ -7,7 +7,7 @@ use std::fmt;
 use std::time::Duration;
 
 use bitvec::{bitvec, field::BitField, order::Lsb0, vec::BitVec, view::BitView};
-use nusb::DeviceInfo;
+use nusb::{DeviceInfo, MaybeFuture};
 use probe_rs_target::ScanChainElement;
 
 use self::{commands::Speed, usb_interface::WchLinkUsbDevice};
@@ -530,6 +530,7 @@ fn get_wlink_info(device: &DeviceInfo) -> Option<DebugProbeInfo> {
             device.serial_number().map(|s| s.to_string()),
             &WchLinkFactory,
             None,
+            false,
         ))
     } else {
         None
@@ -539,8 +540,12 @@ fn get_wlink_info(device: &DeviceInfo) -> Option<DebugProbeInfo> {
 #[tracing::instrument(skip_all)]
 fn list_wlink_devices() -> Vec<DebugProbeInfo> {
     tracing::debug!("Searching for WCH-Link(RV) probes");
-    let Ok(devices) = nusb::list_devices() else {
-        return vec![];
+    let devices = match nusb::list_devices().wait() {
+        Ok(devices) => devices,
+        Err(e) => {
+            tracing::warn!("error listing WCH-Link devices: {e}");
+            return vec![];
+        }
     };
     let probes: Vec<_> = devices
         .filter(|device| device.vendor_id() == VENDOR_ID && device.product_id() == PRODUCT_ID)
