@@ -1,3 +1,4 @@
+use crate::memory::{Operation, OperationKind};
 use crate::rtt::Error;
 use crate::{Core, MemoryInterface};
 use probe_rs_target::RegionMergeIterator;
@@ -24,10 +25,10 @@ pub trait RttChannel {
 pub(crate) struct RttChannelBufferInner<T> {
     standard_name_pointer: T,
     buffer_start_pointer: T,
-    size_of_buffer: T,
-    write_offset: T,
-    read_offset: T,
-    flags: T,
+    size_of_buffer: u32,
+    write_offset: u32,
+    read_offset: u32,
+    flags: u32,
 }
 
 impl<T> RttChannelBufferInner<T> {
@@ -92,8 +93,8 @@ impl RttChannelBuffer {
 
     pub fn size_of_buffer(&self) -> u64 {
         match self {
-            RttChannelBuffer::Buffer32(x) => u64::from(x.size_of_buffer),
-            RttChannelBuffer::Buffer64(x) => x.size_of_buffer,
+            RttChannelBuffer::Buffer32(x) => x.size_of_buffer.into(),
+            RttChannelBuffer::Buffer64(x) => x.size_of_buffer.into(),
         }
     }
 
@@ -106,9 +107,9 @@ impl RttChannelBuffer {
                 (u64::from(block[0]), u64::from(block[1]))
             }
             RttChannelBuffer::Buffer64(h64) => {
-                let mut block = [0u64; 2];
-                core.read_64(ptr + h64.write_buffer_ptr_offset() as u64, block.as_mut())?;
-                (block[0], block[1])
+                let mut block = [0u32; 2];
+                core.read_32(ptr + h64.write_buffer_ptr_offset() as u64, block.as_mut())?;
+                (u64::from(block[0]), u64::from(block[1]))
             }
         })
     }
@@ -133,24 +134,17 @@ impl RttChannelBuffer {
         Ok(())
     }
 
-    pub fn write_read_buffer_ptr(
-        &self,
-        core: &mut Core,
-        ptr: u64,
-        buffer_ptr: u64,
-    ) -> Result<(), Error> {
+    fn write_read_buffer_ptr_operation(&self, ptr: u64, buffer_ptr: u64) -> Operation<'_> {
         match self {
-            RttChannelBuffer::Buffer32(h32) => {
-                core.write_word_32(
-                    ptr + h32.read_buffer_ptr_offset() as u64,
-                    buffer_ptr.try_into().unwrap(),
-                )?;
-            }
-            RttChannelBuffer::Buffer64(h64) => {
-                core.write_word_64(ptr + h64.read_buffer_ptr_offset() as u64, buffer_ptr)?;
-            }
-        };
-        Ok(())
+            RttChannelBuffer::Buffer32(h32) => Operation::new(
+                ptr + h32.read_buffer_ptr_offset() as u64,
+                OperationKind::WriteWord32(buffer_ptr.try_into().unwrap()),
+            ),
+            RttChannelBuffer::Buffer64(h64) => Operation::new(
+                ptr + h64.read_buffer_ptr_offset() as u64,
+                OperationKind::WriteWord64(buffer_ptr),
+            ),
+        }
     }
 
     pub fn read_flags(&self, core: &mut Core, ptr: u64) -> Result<u64, Error> {
@@ -159,7 +153,7 @@ impl RttChannelBuffer {
                 u64::from(core.read_word_32(ptr + h32.flags_offset() as u64)?)
             }
             RttChannelBuffer::Buffer64(h64) => {
-                core.read_word_64(ptr + h64.flags_offset() as u64)?
+                u64::from(core.read_word_32(ptr + h64.flags_offset() as u64)?)
             }
         })
     }
@@ -170,7 +164,7 @@ impl RttChannelBuffer {
                 core.write_word_32(ptr + h32.flags_offset() as u64, flags.try_into().unwrap())?;
             }
             RttChannelBuffer::Buffer64(h64) => {
-                core.write_word_64(ptr + h64.flags_offset() as u64, flags)?;
+                core.write_word_32(ptr + h64.flags_offset() as u64, flags.try_into().unwrap())?;
             }
         };
         Ok(())
@@ -180,14 +174,13 @@ impl RttChannelBuffer {
 #[derive(Debug)]
 pub(crate) struct Channel {
     number: usize,
-    core_id: usize,
     name: Option<String>,
     metadata_ptr: u64,
     info: RttChannelBuffer,
     last_read_ptr: Option<u64>,
 }
 
-// Chanels must follow this data layout when reading/writing memory in order to be compatible with
+// Channels must follow this data layout when reading/writing memory in order to be compatible with
 // the official RTT implementation.
 //
 // struct Channel {
@@ -215,7 +208,6 @@ impl Channel {
 
         let mut this = Channel {
             number,
-            core_id: core.id(),
             metadata_ptr,
             name: None,
             info,
@@ -237,15 +229,6 @@ impl Channel {
         Ok(Some(this))
     }
 
-    /// Validate that the Core id of a request is the same as the Core id against which the Channel was created.
-    pub(crate) fn validate_core_id(&self, core: &mut Core) -> Result<(), Error> {
-        if core.id() != self.core_id {
-            return Err(Error::IncorrectCoreSpecified(self.core_id, core.id()));
-        }
-
-        Ok(())
-    }
-
     pub fn name(&self) -> Option<&str> {
         self.name.as_deref()
     }
@@ -258,7 +241,6 @@ impl Channel {
     ///
     /// See [`ChannelMode`] for more information on what the modes mean.
     pub fn mode(&self, core: &mut Core) -> Result<ChannelMode, Error> {
-        self.validate_core_id(core)?;
         let flags = self.info.read_flags(core, self.metadata_ptr)?;
 
         ChannelMode::try_from(flags & ChannelMode::MASK)
@@ -269,7 +251,6 @@ impl Channel {
     /// See [`ChannelMode`] for more information on what the modes mean.
     pub fn set_mode(&self, core: &mut Core, mode: ChannelMode) -> Result<(), Error> {
         tracing::debug!("Setting RTT channel {} mode to {:?}", self.number, mode);
-        self.validate_core_id(core)?;
         let flags = self.info.read_flags(core, self.metadata_ptr)?;
 
         let new_flags = ChannelMode::set(mode, flags);
@@ -279,8 +260,6 @@ impl Channel {
     }
 
     fn read_pointers(&self, core: &mut Core, channel_kind: &str) -> Result<(u64, u64), Error> {
-        self.validate_core_id(core)?;
-
         let (write, read) = self.info.read_buffer_offsets(core, self.metadata_ptr)?;
 
         // Validate whether the buffers are sensible
@@ -365,7 +344,12 @@ impl UpChannel {
         self.0.set_mode(core, mode)
     }
 
-    fn read_core(&mut self, core: &mut Core, mut buf: &mut [u8]) -> Result<(u64, usize), Error> {
+    fn read_core(
+        &mut self,
+        core: &mut Core,
+        mut buf: &mut [u8],
+        consume: bool,
+    ) -> Result<usize, Error> {
         let (write, mut read) = self.0.read_pointers(core, "up ")?;
 
         let mut total = 0;
@@ -377,6 +361,8 @@ impl UpChannel {
             }
         }
 
+        let mut operations = Vec::with_capacity(3);
+
         // Read while buffer contains data and output buffer has space (maximum of two iterations)
         while !buf.is_empty() {
             let count = min(self.readable_contiguous(write, read), buf.len());
@@ -384,7 +370,9 @@ impl UpChannel {
                 break;
             }
 
-            core.read(self.0.info.buffer_start_pointer() + read, &mut buf[..count])?;
+            let address = self.0.info.buffer_start_pointer() + read;
+            let (buffer, remaining) = buf.split_at_mut(count);
+            operations.push(Operation::new(address, OperationKind::Read(buffer)));
 
             total += count;
             read += count as u64;
@@ -394,11 +382,32 @@ impl UpChannel {
                 read = 0;
             }
 
-            buf = &mut buf[count..];
+            buf = remaining;
         }
-        self.0.last_read_ptr = Some(read);
 
-        Ok((read, total))
+        if consume && total > 0 {
+            // Write read pointer back to target if something was read
+            operations.push(
+                self.0
+                    .info
+                    .write_read_buffer_ptr_operation(self.0.metadata_ptr, read),
+            );
+        }
+
+        core.execute_memory_operations(&mut operations);
+
+        for op in operations.into_iter() {
+            if let Some(result) = op.result {
+                result?;
+            }
+        }
+
+        if consume {
+            // Pointer was successfully written, update stored value
+            self.0.last_read_ptr = Some(read);
+        }
+
+        Ok(total)
     }
 
     /// Reads some bytes from the channel to the specified buffer and returns how many bytes were
@@ -407,16 +416,7 @@ impl UpChannel {
     /// This method will not block waiting for data in the target buffer, and may read less bytes
     /// than would fit in `buf`.
     pub fn read(&mut self, core: &mut Core, buf: &mut [u8]) -> Result<usize, Error> {
-        let (read, total) = self.read_core(core, buf)?;
-
-        if total > 0 {
-            // Write read pointer back to target if something was read
-            self.0
-                .info
-                .write_read_buffer_ptr(core, self.0.metadata_ptr, read)?;
-        }
-
-        Ok(total)
+        self.read_core(core, buf, true)
     }
 
     /// Peeks at the current data in the channel buffer, copies data into the specified buffer and
@@ -425,7 +425,7 @@ impl UpChannel {
     /// The difference from [`read`](UpChannel::read) is that this does not discard the data in the
     /// buffer.
     pub fn peek(&mut self, core: &mut Core, buf: &mut [u8]) -> Result<usize, Error> {
-        Ok(self.read_core(core, buf)?.1)
+        self.read_core(core, buf, false)
     }
 
     /// Calculates amount of contiguous data available for reading
@@ -553,6 +553,7 @@ fn read_c_string(core: &mut Core, ptr: NonZeroU64) -> Result<Option<String>, Err
         )));
     };
 
+    tracing::trace!("read_c_string() ptr = {ptr:#X}");
     // Read up to 128 bytes not going past the end of the region
     let mut bytes = vec![0u8; min(128, (range.end - ptr) as usize)];
     core.read(ptr, bytes.as_mut())?;
